// Copyright (C) 2020 VIRNECT CO., LTD.
// All rights reserved.

#pragma once
#include <string>
#include <vector>

#ifdef EXPORT_API
#if (defined _WIN32 || defined WINCE || defined __CYGWIN__)
#define VARAPI_EXPORT __declspec(dllexport)
#elif defined __GNUC__ && __GNUC__ >= 4 || defined(__APPLE__) || defined(__clang__)
#define VARAPI_EXPORT __attribute__((visibility("default")))
#endif
#else
#define VARAPI_EXPORT
#endif

namespace VIRNECT {

/**
 * @brief Type of target
 */
enum class TargetType
{
    UNDEFINED = 0, ///< Target type is not known
    IMAGE = 1,     ///< Target type is Image
    QRCODE = 2,    ///< Target type is QR code
    CADMODEL = 3,  ///< Target type is CAD model
    MAP = 4        ///< Target type is recorded MAP data
};

[[maybe_unused]] static const char* TargetTypeNames[5] = { "UNDEFINED", "IMAGE", "QRCODE", "CADMODEL", "MAP" }; ///< Names of target types

/**
 * @brief Tracking state of target
 */
enum class TrackingState
{
    NOT_TRACKED = 0, ///< Target is not tracked
    TRACKED = 1      ///< Target is tracked
};

[[maybe_unused]] static const char* TrackingStateNames[2] = { "NOT_TRACKED", "TRACKED" }; ///< Names of tracking states

/**
 * @brief Framework state
 */
enum class FrameworkState
{
    NOT_STARTED = 0, ///< Framework is not started
    RUNNING = 1,     ///< Framework is running now
    PAUSED = 2,      ///< Framework is paused
    FINISHED = 3     ///< Framework ended
};

[[maybe_unused]] static const char* FrameworkStateNames[4] = { "NOT_STARTED", "RUNNING", "PAUSED", "FINISHED" }; ///< Names of framework states

/**
 * @brief Intrinsic camera calibration
 * The Camera calibration model is defined by the intrinsic camera parameters:
 * - Resolution: used during Calibration
 * - Focal length: focal length of the lense
 * - Principal point: position of point without distortion
 * - Lense distortion: Our lens distortion model is based on the typical radian and tangential distortions.
 *   Parameters [r1, r2, t1, t2, r3], r = radial, t = tangential direction
 */
struct CameraCalibration
{
    int32_t mResolution[2] = { 640, 480 };               ///< Image resolution
    double mFocalLength[2] = { 600.0, 600.0 };           ///< Focal length
    double mPrincipalPoint[2] = { 320.0, 240.0 };        ///< Principal point
    double mDistortion[5] = { 0.0, 0.0, 0.0, 0.0, 0.0 }; ///< Lens distortion coefficients
};

/**
 * @brief Two dimensional vector
 * @tparam PREC Precision (float or double)
 */
template <typename PREC>
struct Vec2
{
    PREC x = 0.0f; ///< X coordinate
    PREC y = 0.0f; ///< Y coordinate
};

/**
 * @brief Three dimensional vector
 * @tparam PREC Precision (float or double)
 */
template <typename PREC>
struct Vec3
{
    PREC x = 0.0f; ///< X coordinate
    PREC y = 0.0f; ///< Y coordinate
    PREC z = 0.0f; ///< Z coordinate
};

/**
 * @brief Four dimensional vector
 * @tparam PREC Precision (float or double)
 */
template <typename PREC>
struct Vec4
{
    PREC x = 0.0f; ///< X coordinate
    PREC y = 0.0f; ///< Y coordinate
    PREC z = 0.0f; ///< Z coordinate
    PREC w = 0.0f; ///< W coordinate
};

/**
 * @brief Minimum and maximum size
 * @tparam PREC Precision (float or double)
 */
template <typename PREC>
struct Dimensions
{
    Vec3<PREC> mMin = { 0.0f, 0.0f, 0.0f }; ///< min components
    Vec3<PREC> mMax = { 0.0f, 0.0f, 0.0f }; ///< max components
};

/**
 * @brief Represents one target as a tracking result
 */
struct ObjectTrackResult
{
    std::string mTargetName = "";                                                ///< Target Name
    TargetType mType = TargetType::UNDEFINED;                                    ///< Image, QRCode, CAD
    TrackingState mStatus = TrackingState::NOT_TRACKED;                          ///< Tracking Status
    uint64_t mFrameID = 0;                                                       ///< FrameID
    Vec3<double> mTranslation;                                                   ///< Translation vector that describes the target position in relation to the camera
    double mRotationMatrix[9] = { 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0 }; ///< 3x3 rotation matrix
};

/**
 * @brief Contains target information
 */
struct TargetInformation
{
    std::string mTargetName = "";                                              ///< Target Name
    TargetType mType = TargetType::UNDEFINED;                                  ///< Image, QRCode, CAD
    Dimensions<double> mDimensions = { { 0.0, 0.0, 0.0 }, { 0.0, 0.0, 0.0 } }; ///< Physical target dimensions in meter
    bool mIgnore = false;                                                      ///< If the target should be ignored during processing
};

/**
 * @brief Image struct to pass input data to the framework
 * @tparam CHANNELS amount of pixel channels (1 or 3)
 */
template <uint8_t CHANNELS>
struct Image
{
    std::vector<uint8_t> mBuffer = std::vector<uint8_t>(); ///< Pixel data buffer
    uint32_t mWidth = 0;                                   ///< Width in pixel
    uint32_t mHeight = 0;                                  ///< Height in pixel
    const uint8_t mChannels = CHANNELS;                    ///< Number of channels
    uint32_t mStride = 0;                                  ///< Buffer row length in bytes
};

using ImageGray = Image<1>; ///< Gray image definition
using ImageRGB = Image<3>;  ///< RGB image definition
using ImageRGBA = Image<4>; ///< RGBA image definition

/**
 * @brief Stores vertex information
 */
template <typename PREC>
struct Vertex
{
    Vec3<PREC> mPos = { 0.0f, 0.0f, 0.0f };        ///< Position
    Vec4<PREC> mRGBA = { 0.0f, 0.0f, 0.0f, 0.0f }; ///< Color (RGBA)
    Vec2<PREC> mTexPos = { 0.0f, 0.0f };           ///< Texture position
    Vec3<PREC> mNormal = { 0.0f, 0.0f, 0.0f };     ///< Surface normal
};

/**
 * @brief Represents a simple 3D model with vertex and index array
 */
struct CADModel
{
    std::vector<Vertex<double>> mVertices = {}; ///< Vertex array
    std::vector<int32_t> mIndices = {};         ///< Index array
};

} // namespace VIRNECT
