// Copyright (C) 2020 VIRNECT CO., LTD.
// All rights reserved.

#pragma once
#include <VARAPI/TrackingFramework/Commons.hpp>
#include <unordered_map>

namespace VIRNECT {

/**
 * VINRECT TRACK API Class
 */
class VARAPI_EXPORT TrackingFramework
{

public:
    /**
     * @brief Default constructor returns a singleton instance of the tracking framework
     */
    TrackingFramework();

    /**
     * @brief Destructor stops and cleans the framework resources
     */
    ~TrackingFramework();

    /**
     * @brief Deleted copy constructor
     */
    TrackingFramework(const TrackingFramework&) = delete;

    /**
     * @brief Deleted copy assignment operator
     */
    TrackingFramework& operator=(const TrackingFramework&) = delete;

    /**
     * @brief Returns the framework version
     * @return framework version
     */
    std::string getVersion() const;

    /**
     * @brief Returns the supported target version
     * @return target version
     */
    std::string getSupportedTargetVersion() const;

    /**
     * @brief Set camera calibration of the framework
     * @param cameraCalibration Camera calibration to use
     * @return Success or failure of setting
     */
    bool setCameraCalibration(const CameraCalibration& cameraCalibration);

    /**
     * @brief Provides the current camera calibration parameters
     * @return Camera calibration of the framework
     */
    CameraCalibration getCameraCalibration();

    /**
     * @brief Set the path to the target binaries.Needs to be set before framework initialization
     * @param path Path to binary file
     * @return Success or failure of setting the path
     */
    bool setTargetDataPath(const std::string& path);

    /**
     * @brief Set the targets to track. Needs to be set before framework initialization
     * @param targetNames  Names of targets to track
     * @return Success or failure of setting the names
     */
    bool setTargetNames(const std::vector<std::string>& targetNames);

    /**
     * @brief Enables or disables the map target generation
     * @param enable State to be set
     * @param targetName name of the map target
     * @param savePath name of the map target
     * @return Success or failure of enabling the feature
     */
    bool enableMapTargetGeneration(bool enable, const std::string& targetName, const std::string& savePath);

    /**
     * @brief Saves the target
     * @return Success or failure of saving the target
     */
    bool saveMapTarget();

    /**
     * @brief Initializes the object tracker framework with the given configuration
     * This needs to be called once before the framework can be started to load the target information
     * @return Success or failure of initializing the framework
     */
    bool initialize();

    /**
     * @brief Starts the Framework and processes the image given by the Image Source
     * Use this function if a Image Source was defined by activateImageSourceFileSequence or activateImageSourceUSBCamera
     * @return Success or failure of image processing
     */
    bool process();

    /**
     * @brief Starts the Framework and processes the given image
     * @param image RGB input image to be processed
     * @return Success or failure of image processing
     */
    bool process(const ImageRGB& image);

    /**
     * @brief Starts the Framework and processes the given image
     * @param image RGBA input image to be processed
     * @return Success or failure of image processing
     */
    bool process(const ImageRGBA& image);

    /**
     * @brief Returns a copy of the last Image that was processed for visualization
     * @return Copy of the last Image that was processed
     */
    ImageRGB getLastProcessedImage() const;

    /**
     * @brief Stops the object tracker framework and all internal threads
     * To continue tracking the process function must be called again
     */
    void stop();

    /**
     * @brief Requests the tracking result of the last frame
     * @return A map of tracking results
     */
    std::unordered_map<std::string, VIRNECT::ObjectTrackResult> getTrackingResult() const;

    /**
     * @brief Requests a specific amount of tracking result of the last frame.
     * Will only return targets with the status TRACKED
     * @param maxCount Amount of TRACKED tracking results
     * @return A vector of up to maxCount TRACKED tracking results
     */
    std::vector<VIRNECT::ObjectTrackResult> getConcurrentTrackingResult(uint8_t maxCount = 3) const;

    /**
     * @brief Requests the map point of target
     * @param targetName Name of map target
     * @return 3D tracking points
     */
    std::vector<Vec3<double>> getMapPoints(const std::string& targetName) const;

    /**
     * @brief Returns version of the map target
     * @param targetName Name of map target
     * @return A version of current map target
     */
    uint64_t getMapVersion(const std::string& targetName) const;

    /**
     * @brief Gets information about the target configuration
     * @return A map of target informations
     */
    std::unordered_map<std::string, VIRNECT::TargetInformation> getTargetInformation() const;

    /**
     * @brief Override information about the target configuration
     * @param information Target information to set
     * @return Success or failure of setting the information
     */
    bool setTargetInformation(const std::unordered_map<std::string, VIRNECT::TargetInformation>& information);

    /**
     * @brief Gets current framework state
     * @return Framework state
     */
    FrameworkState getFrameworkState() const;

    /**
     * @brief Returns a CAD model in case of CAD tracking. Empty model otherwise.
     * @return CAD model of tracked target
     */
    CADModel getCADModel() const;
};

/**
 * Namespace to handle licensing information
 */
namespace LICENSE {

/**
 * @brief Set the license key to enable usage of the Tracking Framework. Will start a server request
 * @param key License key
 * @return Validity of the key
 */
VARAPI_EXPORT bool setLicenseKey(const std::string& key);

} // namespace LICENSE

} // namespace VIRNECT
